"""File"""
import json,codecs,configparser,subprocess,platform,os,glob

def get_all_functions(object):
	"""shaonutil.file.get_all_functions(object/file/class)"""
	return [func for func in dir(object) if callable(getattr(object, func))]

def ConfigSectionMap(Config, section):
    dict1 = {}
    options = Config.options(section)
    for option in options:
        try:
            dict1[option] = Config.get(section, option)
            if dict1[option] == -1:
                DebugPrint("skip: %s" % option)
        except:
            print("exception on %s!" % option)
            dict1[option] = None
    return dict1

class CaseConfigParser(configparser.SafeConfigParser):
    def optionxform(self, optionstr):
        return optionstr

def read_configuration_ini(filename):
	config = configparser.ConfigParser()
	config.readfp(codecs.open(filename, "r", "utf8"))
	return config

def read_safecase_configuration_ini(filename):
	config = CaseConfigParser()
	config.readfp(codecs.open(filename, "r", "utf8"))
	return config


def read_json(filename):
	with codecs.open(filename, "r", encoding="utf-8") as fp:
		data = json.load(fp)
	#Print Formatted Dictionary
	#print(json.dumps(data, indent=4))
	return data

def write_json(obj,filename):
	with codecs.open(filename, "w", encoding='utf-8') as fp:
	    json.dump(obj, fp, indent=1)

def read_file(filename):
	with codecs.open(filename, "r", encoding="utf-8") as file_reader:
		lines = file_reader.readlines()

	ill_chars = ['\r','\n']
	_ = []
	for line in lines:
		for ic in ill_chars:
			line = line.replace(ic,'')
		_.append(line)
	filtered_lines = _
	return filtered_lines

def write_file(filename, strs,mode="w"):
	import codecs
	with codecs.open(filename, mode, encoding='utf-8') as file_appender:
		file_appender.writelines(strs)


def open_file_with_default_app(filepath):
	import subprocess
	if platform.system() == 'Darwin':       # macOS
	    subprocess.call(('open', filepath))
	elif platform.system() == 'Windows':    # Windows
	    os.startfile(filepath)
	elif platform.system() == 'Windows':    # Windows
		subprocess.call(('xdg-open', filepath))
	else:                                   # linux variants
	    subprocess.call(('xdg-open', filepath))

def get_last_file_of_dir(filename):
	list_of_files = glob.glob(filename)
	latest_file = max(list_of_files, key=os.path.getctime)
	return latest_file


if __name__ == '__main__':
	pass



"""Network"""
import urllib.parse
import re


def url_encoding_to_utf_8(url):
	"""url_encoding_to_utf_8(url)"""
	url = urllib.parse.unquote(url)
	return url


def check_valid_url(url):
	regex = re.compile(
	        r'^(?:http|ftp)s?://' # http:// or https://
	        r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|' #domain...
	        r'localhost|' #localhost...
	        r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})' # ...or ip
	        r'(?::\d+)?' # optional port
	        r'(?:/?|[/?]\S+)$', re.IGNORECASE)

	return re.match(regex, url) is not None

if __name__ == '__main__':
	pass


#from distutils.core import setup
from setuptools import setup
from os import path

# read the contents of your README file
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()


setup(
  name = 'shaonutil',
  packages = ['shaonutil'],
  version = '0.0.0.15.1',
  long_description=long_description,
  long_description_content_type='text/markdown',
  author = 'Shaon Majumder',
  author_email = 'smazoomder@gmail.com',
  url = 'https://github.com/ShaonMajumder/shaonutil',
  download_url = 'https://github.com/ShaonMajumder/shaonutil/archive/0.0.0.15.1.tar.gz',
  keywords = ['shaon', 'utility', 'statistics'], 
  classifiers = [],
)


"""Statistics"""
def counter(li,number):
	num = 0
	for c in li:
		if c == number:
			num += 1
	return num

def occurance_dic(li):
	dic = {}
	for number in li:
		dic[number] = counter(li,number)
	return dic

def mean(li):
	"""Avearage or mean of elements - shaonutil.stats.mean(list of numbers)"""
	return sum(li)/len(li)

def median(li):
	"""Median of elements - shaonutil.stats.median(list of numbers)"""
	n = len(li) 
	li.sort() 

	if n % 2 == 0: 
		median1 = li[n//2] 
		median2 = li[n//2 - 1] 
		median = (median1 + median2)/2
	else: 
		median = li[n//2] 

	return median

def mode(li):
	"""Mode of elements - shaonutil.stats.mode(list of numbers)"""
	n = len(li)
	data = occurance_dic(li)
	mode = [k for k, v in data.items() if v == max(list(data.values()))] 
	  
	if len(mode) == n: 
	    raise ValueError('No mode found !')
	else:
	    return mode


if __name__ == '__main__':
	pass